@@ -24,6 +24,10 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include "render.h"
 #include "video.h"
 
+#ifndef A_ITALIC
+#   define A_ITALIC 0
+#endif
+
 static iVisChar outside_;
 
 iDefineTypeConstructionArgs(Canvas, (iInt2 size), size)
