@@ -37,6 +37,7 @@ iDeclareTypeConstruction(PtrArray)
 
 iPtrArray * newPointers_PtrArray    (void *ptr, ...); // NULL-terminated
 
+#define     copy_PtrArray(d)        copy_Array(d)
 #define     isEmpty_PtrArray(d)     isEmpty_Array(d)
 #define     size_PtrArray(d)        size_Array(d)
 
@@ -98,7 +99,7 @@ struct ConstIteratorImpl_PtrArray {
         void * const *value; // pointer to array element
         iArrayConstIterator iter;
     };
-    const void *ptr; // array element
+    void *ptr; // array element
 };
 struct ReverseIteratorImpl_PtrArray {
     union {
@@ -112,7 +113,7 @@ struct ReverseConstIteratorImpl_PtrArray {
         void * const *value; // pointer to array element
         iArrayReverseConstIterator iter;
     };
-    const void *ptr; // array element
+    void *ptr; // array element
 };
 ///@}
 
