@@ -42,6 +42,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.</small>
 #include <uniconv.h>
 #include <uninorm.h>
 #include <unistr.h>
+#include <uniwidth.h>
 #include <ctype.h>
 
 #if !defined (iHaveStrnstr)
@@ -91,6 +92,13 @@ const char *script_Char(iChar d) {
     return scr ? scr->name : "";
 }
 
+int width_Char(iChar d) {
+    if (isEmoji_Char(d)) {
+        return 2;
+    }
+    return uc_width(d, localeCharSet_);
+}
+
 void setLocaleCharSet_String(const char *charSet) {
     const size_t n = sizeof(localeCharSet_);
     strncpy(localeCharSet_, charSet, n);
@@ -179,9 +187,12 @@ iString *newBlock_String(const iBlock *data) {
 }
 
 iString *copy_String(const iString *d) {
-    iString *copy = iMalloc(String);
-    initCopy_Block(&copy->chars, &d->chars);
-    return copy;
+    if (d) {
+        iString *copy = iMalloc(String);
+        initCopy_Block(&copy->chars, &d->chars);
+        return copy;
+    }
+    return new_String();
 }
 
 void delete_String(iString *d) {
@@ -464,7 +475,7 @@ size_t size_String(const iString *d) {
 iString *mid_String(const iString *d, size_t charStartPos, size_t charCount) {
     if (charCount == 0) return new_String();
     const char *chars = constData_Block(&d->chars);
-    iRanges range = { 0, size_Block(&d->chars) };
+    iRanges range = { size_Block(&d->chars), size_Block(&d->chars) };
     size_t pos = 0;
     iConstForEach(String, i, d) {
         if (pos > charStartPos && pos == charStartPos + charCount) {
@@ -486,7 +497,7 @@ iString *mid_String(const iString *d, size_t charStartPos, size_t charCount) {
 iString *upper_String(const iString *d) {
     return upperLang_String(d, currentLocaleLanguage_());
 }
-    
+
 iString *upperLang_String(const iString *d, const char *langCode) {
     size_t len = 0;
     uint8_t *str = u8_toupper((const uint8_t *) cstr_String(d),
@@ -901,7 +912,7 @@ iStringList *split_Rangecc(const iRangecc d, const char *separator) {
 
 int toInt_String(const iString *d) {
     if (startsWith_String(d, "0x") || startsWith_String(d, "0X")) {
-        return strtol(cstr_String(d), NULL, 16);
+        return (int) strtol(cstr_String(d), NULL, 16);
     }
     return atoi(cstr_String(d));
 }
@@ -1097,7 +1108,7 @@ int decodePrecedingBytes_MultibyteChar(const char *bytes, const char *start, iCh
     if (!precPos) {
         return 0;
     }
-    return bytes - precPos;
+    return (int) (bytes - precPos);
 }
 
 static char *threadLocalCharBuffer_(void) {
