@@ -10,6 +10,7 @@ BUILD_LANGS = [ 'en', # base strings
     'eo',
     'es',
     'es_MX',
+    'eu',
     'fi',
     'fr',
     'gl',
@@ -18,6 +19,7 @@ BUILD_LANGS = [ 'en', # base strings
     'ie',
     'isv',
     'it',
+    'ja',
     'nl',
     'pl',
     'ru',
