@@ -21,6 +21,7 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 #include "player.h"
+#include "defs.h"
 #include "buf.h"
 #include "lang.h"
 
@@ -474,12 +475,11 @@ static iRangecc mediaType_(const iString *str) {
     return part;
 }
 
-static iContentSpec contentSpec_Player_(const iPlayer *d) {
+static iContentSpec detectContentSpec_Player_(const iPlayer *d) {
     iContentSpec content;
     iZap(content);
     const size_t dataSize = size_InputBuf(d->data);
-    iBuffer *buf = iClob(new_Buffer());
-    open_Buffer(buf, &d->data->data);
+    iBuffer *buf = NULL;
     const iRangecc mediaType = mediaType_(&d->mime);
     if (equal_Rangecc(mediaType, "audio/wave") || equal_Rangecc(mediaType, "audio/wav") ||
         equal_Rangecc(mediaType, "audio/x-wav") || equal_Rangecc(mediaType, "audio/x-pn-wav")) {
@@ -498,6 +498,12 @@ static iContentSpec contentSpec_Player_(const iPlayer *d) {
         /* TODO: Could try decoders to see if one works? */
         content.type = none_DecoderType;
     }
+    if (content.type != none_DecoderType) {
+        buf = iClob(new_Buffer());
+        /* This holds a reference to the data block. The decoder runs in a background
+           thread so it needs its own copy of the data. */
+        open_Buffer(buf, &d->data->data);
+    }
     if (content.type == wav_DecoderType && dataSize >= 44) {
         /* Read the RIFF/WAVE header. */
         iStream *is = stream_Buffer(buf);
@@ -620,7 +626,7 @@ static iContentSpec contentSpec_Player_(const iPlayer *d) {
     iAssert(content.inputFormat == content.output.format ||
             (content.inputFormat == AUDIO_S24LSB && content.output.format == AUDIO_S16) ||
             (content.inputFormat == AUDIO_F64LSB && content.output.format == AUDIO_F32));
-    content.output.samples = 8192;
+    content.output.samples = isAndroid_Platform() ? content.output.freq / 2 : 8192;
     return content;
 }
 
@@ -712,6 +718,8 @@ void updateSourceData_Player(iPlayer *d, const iString *mimeType, const iBlock *
                 break;
             }
             /* The old parts cannot have changed. */
+            /* NOTE: The decoder will hold a reference to the data block, which means
+               appending here will cause a copy-on-write detach to occur. */
             appendData_Block(&input->data, constBegin_Block(data) + oldSize, newSize - oldSize);
             break;
         }
@@ -742,14 +750,14 @@ size_t sourceDataSize_Player(const iPlayer *d) {
 
 static iBool setupSDLAudio_(iBool init) {
     static iBool isAudioInited_ = iFalse;
-    if (init) {
+    if (init && !isAudioInited_) {
         if (SDL_InitSubSystem(SDL_INIT_AUDIO)) {
             fprintf(stderr, "[SDL] audio init failed: %s\n", SDL_GetError());
             return iFalse;
         }
         isAudioInited_ = iTrue;
     }
-    else if (isAudioInited_) {
+    else if (!init && isAudioInited_ && !isAndroid_Platform()) {
         SDL_QuitSubSystem(SDL_INIT_AUDIO);
         isAudioInited_ = iFalse;
     }
@@ -768,7 +776,7 @@ iBool start_Player(iPlayer *d) {
         return iTrue;
     }
 #endif
-    iContentSpec content = contentSpec_Player_(d);
+    iContentSpec content = detectContentSpec_Player_(d);
     if (!content.output.freq) {
         return iFalse;
     }
