@@ -42,6 +42,7 @@ struct Impl_LabelWidget {
     int     kmods;
     iChar   icon;
     int     forceFg;
+    int     iconColor;
     iString command;
     iClick  click;
     struct {
@@ -53,8 +54,11 @@ struct Impl_LabelWidget {
         uint16_t wrap                : 1;
         uint16_t allCaps             : 1;
         uint16_t removeTrailingColon : 1;
-        uint16_t chevron             : 1;        
+        uint16_t chevron             : 1;
         uint16_t checkMark           : 1;
+        uint16_t truncateToFit       : 1;
+        uint16_t menuCanceling       : 1;
+        uint16_t noLabel             : 1;
     } flags;
 };
 
@@ -68,10 +72,15 @@ static iBool isHover_LabelWidget_(const iLabelWidget *d) {
 static iInt2 padding_LabelWidget_(const iLabelWidget *d, int corner) {
     const iWidget *w = constAs_Widget(d);
     const int64_t flags = flags_Widget(w);
-    const iInt2 widgetPad = (corner   == 0 ? init_I2(w->padding[0], w->padding[1])
-                             : corner == 1 ? init_I2(w->padding[2], w->padding[1])
-                             : corner == 2 ? init_I2(w->padding[2], w->padding[3])
-                             : init_I2(w->padding[0], w->padding[3]));
+    iInt2          widgetPad = (corner == 0   ? init_I2(w->padding[0], w->padding[1])
+                                : corner == 1 ? init_I2(w->padding[2], w->padding[1])
+                                : corner == 2 ? init_I2(w->padding[2], w->padding[3])
+                                              : init_I2(w->padding[0], w->padding[3]));
+    if (d->flags.chevron) {
+        if (corner == 1 || corner == 2) {
+            widgetPad.x += gap_UI * 5;
+        }
+    }
     if (isMobile_Platform()) {
         return add_I2(widgetPad,
                       init_I2(flags & tight_WidgetFlag ? 2 * gap_UI : (4 * gap_UI),
@@ -92,12 +101,16 @@ static iBool checkModifiers_(int have, int req) {
 
 static void trigger_LabelWidget_(const iLabelWidget *d) {
     const iWidget *w = constAs_Widget(d);
-    postCommand_Widget(&d->widget, "%s", cstr_String(&d->command));
+    postCommand_Widget(w, "%s", cstr_String(&d->command));
     if (flags_Widget(w) & radio_WidgetFlag) {
         iForEach(ObjectList, i, children_Widget(w->parent)) {
             setFlags_Widget(i.object, selected_WidgetFlag, d == i.object);
         }
     }
+    /* Triggering a menu item will always close all popup menus. */
+    if (d->flags.menuCanceling) {
+        postCommand_Root(NULL, "menu.cancel");
+    }
 }
 
 static void updateKey_LabelWidget_(iLabelWidget *d) {
@@ -110,6 +123,32 @@ static void updateKey_LabelWidget_(iLabelWidget *d) {
     }
 }
 
+static void endSiblingOrderDrag_LabelWidget_(iLabelWidget *d) {
+    iWidget *w = as_Widget(d);
+    if ((w->flags2 & siblingOrderDraggable_WidgetFlag2) && (flags_Widget(w) & dragged_WidgetFlag)) {
+        float dragAmount = (float) w->visualOffset.to / (float) width_Widget(w);
+        if (dragAmount > -0.5f && dragAmount < -0.1f) {
+            dragAmount = -0.5f;
+        }
+        else if (dragAmount < 0.5f && dragAmount > 0.1f) {
+            dragAmount = 0.5f;
+        }
+        postCommand_Widget(w, "tabs.move arg:%d dragged:1", iRound(dragAmount));
+        setVisualOffset_Widget(w, 0, 0, 0);
+        setFlags_Widget(w, dragged_WidgetFlag | keepOnTop_WidgetFlag, iFalse);
+    }
+}
+
+static iBool isSubmenuItem_LabelWidget_(const iLabelWidget *d) {
+    if (isAndroid_Platform()) {
+        /* On Android, we don't have system menus nor do we want actual submenu popups
+           to appear. The "submenu" command will cause the submenu to open as a normal
+           menu. */
+        return iFalse;
+    }
+    return startsWith_String(&d->command, "submenu id:");
+}
+
 static iBool processEvent_LabelWidget_(iLabelWidget *d, const SDL_Event *ev) {
     iWidget *w = &d->widget;
     if (isMetricsChange_UserEvent(ev)) {
@@ -127,6 +166,14 @@ static iBool processEvent_LabelWidget_(iLabelWidget *d, const SDL_Event *ev) {
         updateKey_LabelWidget_(d);
         return iFalse;
     }
+    else if (isCommand_UserEvent(ev, "cancel")) {
+        if (flags_Widget(w) & pressed_WidgetFlag) {
+            setFlags_Widget(w, pressed_WidgetFlag, iFalse);
+            endSiblingOrderDrag_LabelWidget_(d);
+            refresh_Widget(w);
+        }
+        return iFalse;
+    }
     else if (isCommand_Widget(w, ev, "focus.gained") ||
              isCommand_Widget(w, ev, "focus.lost")) {
         iWidget *scr = findOverflowScrollable_Widget(w);
@@ -165,21 +212,39 @@ static iBool processEvent_LabelWidget_(iLabelWidget *d, const SDL_Event *ev) {
             }
         }
 #endif
+        if (isSubmenuItem_LabelWidget_(d) && ev->type == SDL_MOUSEBUTTONDOWN &&
+                contains_Widget(w, init_I2(ev->button.x, ev->button.y))) {
+            /* Submenus are triggered by hovering over the item. Clicking down nothing. */
+            postCommand_Widget(d, "submenu.open");
+            return iTrue;
+        }
         switch (processEvent_Click(&d->click, ev)) {
             case started_ClickResult:
                 setFlags_Widget(w, pressed_WidgetFlag, iTrue);
                 refresh_Widget(w);
                 return iTrue;
+            case drag_ClickResult:
+                if (w->flags2 & siblingOrderDraggable_WidgetFlag2) {
+                    setFlags_Widget(w, dragged_WidgetFlag | keepOnTop_WidgetFlag, iTrue);
+                    setVisualOffset_Widget(w, delta_Click(&d->click).x, 0, 0);
+                    refresh_Widget(w);
+                    return iTrue;
+                }
+                break;
             case aborted_ClickResult:
                 setFlags_Widget(w, pressed_WidgetFlag, iFalse);
+                endSiblingOrderDrag_LabelWidget_(d);
                 refresh_Widget(w);
                 return iTrue;
 //            case double_ClickResult:
             case finished_ClickResult:
                 setFlags_Widget(w, pressed_WidgetFlag, iFalse);
+                endSiblingOrderDrag_LabelWidget_(d);
                 trigger_LabelWidget_(d);
                 refresh_Widget(w);
-                setFocus_Widget(NULL);
+                if (focus_Widget() == w) {
+                    setFocus_Widget(NULL);
+                }
                 return iTrue;
             default:
                 break;
@@ -210,9 +275,14 @@ static void keyStr_LabelWidget_(const iLabelWidget *d, iString *str) {
 }
 
 static iBool areTabButtonsThemeColored_(void) {
+    if (type_Window(get_Window()) != main_WindowType) {
+        /* Only the main window has themes available, since it has DocumentWidgets. */
+        return iFalse;
+    }
     const enum iGmDocumentTheme docTheme = docTheme_Prefs(prefs_App());
     const iBool isDarkUI = isDark_ColorTheme(colorTheme_App());
     return (docTheme == colorfulLight_GmDocumentTheme ||
+            docTheme == vibrantLight_GmDocumentTheme ||
             docTheme == sepia_GmDocumentTheme ||
             (docTheme == oceanic_GmDocumentTheme && !isDarkUI));
 }
@@ -289,7 +359,7 @@ static void getColors_LabelWidget_(const iLabelWidget *d, int *bg, int *fg, int
                 *frame1 = *bg;
             }
         }
-    }    
+    }
     if (isFocus) {
         *frame1 = *frame2 = (isSel ? uiText_ColorId : uiInputFrameFocused_ColorId);
     }
@@ -300,12 +370,19 @@ static void getColors_LabelWidget_(const iLabelWidget *d, int *bg, int *fg, int
     if (colorEscape == uiTextCaution_ColorId) {
         *icon = *meta = colorEscape;
     }
+    if (d->iconColor != none_ColorId) {
+        *icon = d->iconColor;
+        if ((*icon >= brown_ColorId && *icon <= blue_ColorId) && !isDarkTheme) {
+            /* Auto-adjust absolute color IDs to suit the UI theme. */
+            (*icon)--; /* make it darker */
+        }
+    }
     if (isHover) {
         if (isFrameless) {
             if (prefs_App()->accent == gray_ColorAccent && prefs_App()->theme >= light_ColorTheme) {
                 *bg = gray75_ColorId;
             }
-            else {
+            else if (!isSel) {
                 *bg = uiBackgroundFramelessHover_ColorId;
             }
             *fg = uiTextFramelessHover_ColorId;
@@ -395,11 +472,18 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
     const enum iColorId colorEscape = parseEscape_Color(cstr_String(&d->label), NULL);
     const iBool isCaution = (colorEscape == uiTextCaution_ColorId);
     if (bg >= 0) {
+        if (flags & dragged_WidgetFlag) {
+            p.alpha = 0x70;
+            SDL_SetRenderDrawBlendMode(renderer_Window(get_Window()), SDL_BLENDMODE_BLEND);
+        }
         fillRect_Paint(&p, rect, bg);
+        if (flags & dragged_WidgetFlag) {
+            SDL_SetRenderDrawBlendMode(renderer_Window(get_Window()), SDL_BLENDMODE_NONE);
+        }
     }
     if (isFocused_Widget(w)) {
         iRect frameRect = adjusted_Rect(rect, zero_I2(), init1_I2(-1));
-        drawRectThickness_Paint(&p, frameRect, gap_UI / 4, uiTextAction_ColorId /*frame*/);        
+        drawRectThickness_Paint(&p, frameRect, gap_UI / 4, uiTextAction_ColorId /*frame*/);
     }
     else if (~flags & frameless_WidgetFlag) {
         iRect frameRect = adjusted_Rect(rect, zero_I2(), init1_I2(-1));
@@ -414,7 +498,7 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
 #if SDL_COMPILEDVERSION == SDL_VERSIONNUM(2, 0, 16)
             if (isOpenGLRenderer_Window()) {
                 /* A very curious regression in SDL 2.0.16. */
-                points[3].x--;    
+                points[3].x--;
             }
 #endif
             if (d->flags.noBottomFrame && !isFocused_Widget(w) && !isHover) {
@@ -460,8 +544,19 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
         draw_WrapText(&wt, d->font, topLeft_Rect(cont), fg);
     }
     else if (flags & alignLeft_WidgetFlag) {
-        draw_Text(d->font, add_I2(bounds.pos, addX_I2(padding_LabelWidget_(d, 0), iconPad)),
-                  fg, "%s", cstr_String(&d->label));
+        const iInt2 topLeft = add_I2(bounds.pos, addX_I2(padding_LabelWidget_(d, 0), iconPad));
+        if (d->flags.truncateToFit) {
+            const char *endPos;
+            tryAdvanceNoWrap_Text(d->font,
+                                  range_String(&d->label),
+                                  width_Rect(rect) - padding_LabelWidget_(d, 0).x -
+                                      padding_LabelWidget_(d, 1).x - iconPad,
+                                  &endPos);
+            drawRange_Text(d->font, topLeft, fg, (iRangecc){ constBegin_String(&d->label), endPos });
+        }
+        else {
+            draw_Text(d->font, topLeft, fg, "%s", cstr_String(&d->label));
+        }
         if ((flags & drawKey_WidgetFlag) && d->key) {
             iString str;
             init_String(&str);
@@ -470,11 +565,7 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
                            add_I2(topRight_Rect(bounds),
                                   addX_I2(negX_I2(padding_LabelWidget_(d, 1)),
                                           deviceType_App() == tablet_AppDeviceType ? gap_UI : 0)),
-                           metaColor,/*
-                           isHover || flags & pressed_WidgetFlag ? fg
-//                           : isCaution                ? uiTextCaution_ColorId
-                           : colorEscape != none_ColorId ? colorEscape
-                                                      : uiTextShortcut_ColorId,*/
+                           metaColor,
                            right_Alignment,
                            "%s",
                            cstr_String(&str));
@@ -496,7 +587,7 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
             moved_Rect(
                 adjusted_Rect(bounds,
                               init_I2(iconPad * (flags & tight_WidgetFlag ? 1.0f : 1.5f), 0),
-                          init_I2(-iconPad * (flags & tight_WidgetFlag ? 0.5f : 1.0f), 0)),
+                              init_I2(-iconPad * (flags & tight_WidgetFlag ? 0.5f : 1.0f), 0)),
                 d->labelOffset),
             d->flags.alignVisual,
             d->flags.drawAsOutline ? fg : none_ColorId,
@@ -508,11 +599,17 @@ static void draw_LabelWidget_(const iLabelWidget *d) {
         const iRect chRect = rect;
         const int chSize = lineHeight_Text(d->font);
         int offset = 0;
-        if (d->flags.chevron) {
-            offset = -iconPad;
+        if (isMobile_Platform()) {
+            /* These are used in the sub-panel buttons. */
+            if (d->flags.chevron && iconPad) {
+                offset = -iconPad;
+            }
+            else {
+                offset = -10 * gap_UI;
+            }
         }
         else {
-            offset = -10 * gap_UI;
+            offset = -6 * gap_UI;
         }
         drawCentered_Text(d->font,
                           (iRect){ addX_I2(topRight_Rect(chRect), offset),
@@ -543,8 +640,14 @@ static void sizeChanged_LabelWidget_(iLabelWidget *d) {
 iInt2 defaultSize_LabelWidget(const iLabelWidget *d) {
     const iWidget *w = constAs_Widget(d);
     const int64_t flags = flags_Widget(w);
-    iInt2 size = add_I2(measure_Text(d->font, cstr_String(&d->label)).bounds.size,
-                        add_I2(padding_LabelWidget_(d, 0), padding_LabelWidget_(d, 2)));
+    iInt2 size;
+    if (!d->flags.noLabel) {
+        size = add_I2(measure_Text(d->font, cstr_String(&d->label)).bounds.size,
+                      add_I2(padding_LabelWidget_(d, 0), padding_LabelWidget_(d, 2)));
+    }
+    else {
+        size = zero_I2();
+    }
     if ((flags & drawKey_WidgetFlag) && d->key) {
         iString str;
         init_String(&str);
@@ -593,11 +696,19 @@ void init_LabelWidget(iLabelWidget *d, const char *label, const char *cmd) {
     iZap(d->flags);
     d->font = uiLabel_FontId;
     d->forceFg = none_ColorId;
+    d->iconColor = none_ColorId;
     d->icon = 0;
     d->labelOffset = zero_I2();
-    initCStr_String(&d->srcLabel, label);
-    initCopy_String(&d->label, &d->srcLabel);
-    replaceVariables_LabelWidget_(d);
+    if (label) {
+        initCStr_String(&d->srcLabel, label);
+        initCopy_String(&d->label, &d->srcLabel);
+        replaceVariables_LabelWidget_(d);
+    }
+    else {
+        d->flags.noLabel = iTrue;
+        init_String(&d->srcLabel);
+        init_String(&d->label);
+    }
     if (cmd) {
         initCStr_String(&d->command, cmd);
     }
@@ -683,6 +794,10 @@ void setWrap_LabelWidget(iLabelWidget *d, iBool wrap) {
     d->flags.wrap = wrap;
 }
 
+void setTruncateToFit_LabelWidget (iLabelWidget *d, iBool truncateToFit) {
+    d->flags.truncateToFit = truncateToFit;
+}
+
 void setOutline_LabelWidget(iLabelWidget *d, iBool drawAsOutline) {
     if (d) {
         d->flags.drawAsOutline = drawAsOutline;
@@ -703,6 +818,12 @@ void setRemoveTrailingColon_LabelWidget(iLabelWidget *d, iBool removeTrailingCol
     }
 }
 
+void setMenuCanceling_LabelWidget(iLabelWidget *d, iBool menuCanceling) {
+    if (d) {
+        d->flags.menuCanceling = menuCanceling;
+    }
+}
+
 void setTextOffset_LabelWidget(iLabelWidget *d, iInt2 offset) {
     d->labelOffset = offset;
 }
@@ -739,12 +860,19 @@ void setIcon_LabelWidget(iLabelWidget *d, iChar icon) {
     }
 }
 
+void setIconColor_LabelWidget(iLabelWidget *d, int color) {
+    d->iconColor = color;
+}
+
 iBool checkIcon_LabelWidget(iLabelWidget *d) {
     if (isEmpty_String(&d->label)) {
         d->icon = 0;
         return iFalse;
     }
     d->icon = removeIconPrefix_String(&d->label);
+    if (isTerminal_Platform() && prefs_App()->simpleChars) {
+        d->icon = 0; /* ASCII icons may get confused with key shortcuts */
+    }
     return d->icon != 0;
 }
 
