@@ -32,12 +32,15 @@ iDeclareObjectConstructionArgs(LabelWidget, const char *label, const char *comma
 void    setAlignVisually_LabelWidget(iLabelWidget *, iBool alignVisual);
 void    setNoAutoMinHeight_LabelWidget  (iLabelWidget *, iBool noAutoMinHeight);
 void    setNoTopFrame_LabelWidget   (iLabelWidget *, iBool noTopFrame);
+void    setNoBottomFrame_LabelWidget(iLabelWidget *, iBool noBottomFrame);
 void    setChevron_LabelWidget      (iLabelWidget *, iBool chevron);
 void    setCheckMark_LabelWidget    (iLabelWidget *, iBool checkMark);
 void    setWrap_LabelWidget         (iLabelWidget *, iBool wrap);
+void    setTruncateToFit_LabelWidget(iLabelWidget *, iBool truncateToFit);
 void    setOutline_LabelWidget      (iLabelWidget *, iBool drawAsOutline);
 void    setAllCaps_LabelWidget      (iLabelWidget *, iBool allCaps);
 void    setRemoveTrailingColon_LabelWidget  (iLabelWidget *, iBool removeTrailingColon);
+void    setMenuCanceling_LabelWidget(iLabelWidget *, iBool menuCanceling);
 void    setTextOffset_LabelWidget   (iLabelWidget *, iInt2 offset);
 void    setFont_LabelWidget         (iLabelWidget *, int fontId);
 void    setTextColor_LabelWidget    (iLabelWidget *, int color);
@@ -45,6 +48,7 @@ void    setText_LabelWidget         (iLabelWidget *, const iString *text); /* re
 void    setTextCStr_LabelWidget     (iLabelWidget *, const char *text);
 void    setCommand_LabelWidget      (iLabelWidget *, const iString *command);
 void    setIcon_LabelWidget         (iLabelWidget *, iChar icon);
+void    setIconColor_LabelWidget    (iLabelWidget *, int color);
 
 iBool   checkIcon_LabelWidget       (iLabelWidget *);
 void    updateSize_LabelWidget      (iLabelWidget *);
@@ -58,6 +62,7 @@ int             font_LabelWidget        (const iLabelWidget *);
 const iString * text_LabelWidget        (const iLabelWidget *);
 const iString * sourceText_LabelWidget  (const iLabelWidget *); /* untranslated */
 const iString * command_LabelWidget     (const iLabelWidget *);
+int             textColor_LabelWidget   (const iLabelWidget *);
 iChar           icon_LabelWidget        (const iLabelWidget *);
 iBool           isWrapped_LabelWidget   (const iLabelWidget *);
 
