@@ -29,6 +29,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include "inputwidget.h"
 #include "labelwidget.h"
 #include "root.h"
+#include "snippetwidget.h"
 #include "text.h"
 #include "widget.h"
 #include "window.h"
@@ -43,7 +44,7 @@ const iToolbarActionSpec toolbarActions_Mobile[max_ToolbarAction] = {
     { home_Icon, "${menu.home}", "navigate.home" },
     { upArrow_Icon, "${menu.parent}", "navigate.parent" },
     { reload_Icon, "${menu.reload}", "navigate.reload" },
-    { add_Icon, "${menu.newtab}", "tabs.new" },
+    { add_Icon, "${menu.newtab}", "tabs.new append:1" },
     { close_Icon, "${menu.closetab}", "tabs.close" },
     { bookmark_Icon, "${menu.page.bookmark}", "bookmark.add" },
     { globe_Icon, "${menu.page.translate}", "document.translate" },
@@ -51,7 +52,10 @@ const iToolbarActionSpec toolbarActions_Mobile[max_ToolbarAction] = {
     { edit_Icon, "${menu.page.upload.edit}", "document.upload copy:1" },
     { magnifyingGlass_Icon, "${menu.find}", "focus.set id:find.input" },
     { gear_Icon, "${menu.settings}", "preferences" },
-    { leftHalf_Icon, "${menu.sidebar.left}", "sidebar.toggle" },        
+    { leftHalf_Icon, "${menu.sidebar.left}", "sidebar.toggle" },
+    { rightHalf_Icon, "${menu.sidebar.right}", "sidebar2.toggle" },
+    { upArrowBar_Icon, "${keys.top}", "scroll.top smooth:1" },
+    { downArrowBar_Icon, "${keys.bottom}", "scroll.bottom smooth:1" },
 };
 
 iBool isUsingPanelLayout_Mobile(void) {
@@ -88,7 +92,8 @@ iBool isFullSizePanel_Mobile(const iWidget *panels) {
         return !iCmpStr(id, "prefs") || !iCmpStr(id, "upload");
     }
     return !iCmpStr(id, "prefs") || startsWith_CStr(id, "bmed") || startsWith_CStr(id, "sitespec ")
-        || !iCmpStr(id, "upload") || !iCmpStr(id, "certimport") || !iCmpStr(id, "ident");
+        || !iCmpStr(id, "upload") || !iCmpStr(id, "certimport") || !iCmpStr(id, "ident")
+        || !iCmpStr(id, "snip");
 }
 
 iLocalDef iBool isFullSizePanel_(const iWidget *panels) {
@@ -144,7 +149,7 @@ static iWidget *findTitleLabel_(iWidget *panel) {
     return NULL;
 }
 
-static void updateCertListHeight_(iWidget *detailStack) {
+static void updateListHeights_(iWidget *detailStack) {
     iWidget *certList = findChild_Widget(detailStack, "certlist");
     if (certList) {
         setFixedSize_Widget(certList,
@@ -152,6 +157,13 @@ static void updateCertListHeight_(iWidget *detailStack) {
                                     -1 * gap_UI + bottom_Rect(safeRect_Root(certList->root)) -
                                         top_Rect(boundsWithoutVisualOffset_Widget(certList))));
     }
+    iWidget *snippetList = findChild_Widget(detailStack, "sniped");
+    if (snippetList) {
+        setFixedSize_Widget(snippetList,
+                            init_I2(-1,
+                                    -1 * gap_UI + bottom_Rect(safeRect_Root(certList->root)) -
+                                    top_Rect(boundsWithoutVisualOffset_Widget(certList))));
+    }
 }
 
 static iBool mainDetailSplitHandler_(iWidget *mainDetailSplit, const char *cmd) {
@@ -188,13 +200,14 @@ static iBool mainDetailSplitHandler_(iWidget *mainDetailSplit, const char *cmd)
         iForEach(ObjectList, i, children_Widget(detailStack)) {
             iWidget *panel = i.object;
             setFlags_Widget(panel, leftEdgeDraggable_WidgetFlag, !isSideBySide);
+            setFlags_Widget(panel, focusRoot_WidgetFlag, !isSideBySide);
             if (isSideBySide) {
                 setVisualOffset_Widget(panel, 0, 0, 0);
             }
             setPadding_Widget(panel, pad, 0, pad, pad + bottomSafeInset_Mobile());
         }
         arrange_Widget(mainDetailSplit);
-        updateCertListHeight_(detailStack);
+        updateListHeights_(detailStack);
     }
     else if (deviceType_App() == tablet_AppDeviceType && equal_Command(cmd, "keyboard.changed")) {
         if (arg_Command(cmd) > 0 && !isFullSizePanel_(sheet)) {
@@ -237,7 +250,13 @@ static void updateNaviActionVisibility_(iWidget *sheet, iWidget *curPanel) {
                         userData_Object(i.object) && userData_Object(i.object) != curPanel);
     }
     arrange_Widget(navi);
-    refresh_Widget(navi);    
+    refresh_Widget(navi);
+    iForEach(ObjectList, j, children_Widget(naviActions)) {
+        iWidget *menu = findChild_Widget(j.object, "menu");
+        if (menu && flags_Widget(menu) & nativeMenu_WidgetFlag) {
+            updateAfterBoundsChange_SystemMenu(menu);
+        }
+    }
 }
 
 static iBool topPanelHandler_(iWidget *topPanel, const char *cmd) {
@@ -265,10 +284,10 @@ static iBool topPanelHandler_(iWidget *topPanel, const char *cmd) {
             iLabelWidget *naviTitle = findChild_Widget(sheet, "navi.title");
             updateText_LabelWidget(naviTitle, text_LabelWidget((iLabelWidget *) findTitleLabel_(panel)));
             updateNaviActionVisibility_(sheet, panel);
-        }        
+        }
         setFlags_Widget(button, selected_WidgetFlag, iTrue);
         postCommand_Widget(topPanel, "panel.changed arg:%d", panelIndex);
-        updateCertListHeight_(findDetailStack_(topPanel));
+        updateListHeights_(findDetailStack_(topPanel));
         return iTrue;
     }
     if (equal_Command(cmd, "swipe.back")) {
@@ -300,7 +319,8 @@ static iBool topPanelHandler_(iWidget *topPanel, const char *cmd) {
             if (findWidget_App("ident")) {
                 postCommand_App("ident.cancel");
             }
-            else if ((widget = findWidget_App("certimport")) != NULL) {
+            else if ((widget = findWidget_App("certimport")) != NULL ||
+                     (widget = findWidget_App("snip")) != NULL) {
                 postCommand_Widget(widget, "cancel");
             }
             else if (findWidget_App("prefs")) {
@@ -309,8 +329,7 @@ static iBool topPanelHandler_(iWidget *topPanel, const char *cmd) {
             else if (findWidget_App("upload")) {
                 postCommand_App("upload.cancel");
             }
-            else if (findWidget_App("bmed.sidebar") || findWidget_App("bmed.create") ||
-                     findWidget_App("bmed")) {
+            else if (findWidget_App("bmed.title")) {
                 postCommand_App("bmed.cancel");
             }
             else if (findWidget_App("ident")) {
@@ -451,7 +470,7 @@ static iWidget *makeValuePadding_(iWidget *value) {
     iInputWidget *input = isInstance_Object(value, &Class_InputWidget) ? (iInputWidget *) value : NULL;
     if (input) {
         setFont_InputWidget(input, labelFont_());
-        setContentPadding_InputWidget(input, 3 * gap_UI, 3 * gap_UI);
+        setContentPadding_InputWidget(input, 2 * gap_UI, 3 * gap_UI);
     }
     iWidget *pad = new_Widget();
     setBackgroundColor_Widget(pad, uiBackgroundSidebar_ColorId);
@@ -473,7 +492,7 @@ static iWidget *makeValuePaddingWithHeading_(iLabelWidget *heading, iWidget *val
                     resizeWidthOfChildren_WidgetFlag |
                     arrangeHorizontal_WidgetFlag, iTrue);
     setBackgroundColor_Widget(div, uiBackgroundSidebar_ColorId);
-    setPadding_Widget(div, gap_UI, gap_UI, 4 * gap_UI, gap_UI);
+    setPadding_Widget(div, gap_UI, gap_UI, !isInput ? 4 * gap_UI : (2 * gap_UI), gap_UI);
     addChildFlags_Widget(div, iClob(heading), 0);
     setPadding1_Widget(as_Widget(heading), 0);
     setFont_LabelWidget(heading, labelFont_());
@@ -491,7 +510,10 @@ static iWidget *makeValuePaddingWithHeading_(iLabelWidget *heading, iWidget *val
 //                                           value));
     }
     else {
-        addChildFlags_Widget(div, iClob(new_Widget()), expand_WidgetFlag);
+        setFlags_Widget(as_Widget(heading),
+                        fixedHeight_WidgetFlag /* for automatic wrap height */ |
+                        expand_WidgetFlag, iTrue);
+        setWrap_LabelWidget(heading, iTrue);
         addChild_Widget(div, iClob(value));
     }
 //    printTree_Widget(div);
@@ -521,6 +543,7 @@ static iWidget *addChildPanel_(iWidget *parent, iLabelWidget *panelButton,
                              arrangeHeight_WidgetFlag | overflowScrollable_WidgetFlag |
                              drawBackgroundToBottom_WidgetFlag |
                              horizontalOffset_WidgetFlag | commandOnClick_WidgetFlag);
+    iAssert(~flags_Widget(panel) & unhittable_WidgetFlag);
     return panel;
 }
 
@@ -591,7 +614,7 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
         setRemoveTrailingColon_LabelWidget(heading, iTrue);
         addChild_Widget(panel, iClob(heading));
         setId_Widget(as_Widget(heading), id);
-    }    
+    }
     else if (equal_Command(spec, "toggle")) {
         iLabelWidget *toggle = (iLabelWidget *) makeToggle_Widget(id);
         setFont_LabelWidget(toggle, labelFont_());
@@ -608,7 +631,12 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
                         alignRight_WidgetFlag | noBackground_WidgetFlag |
                             frameless_WidgetFlag, iTrue);
         setId_Widget(as_Widget(drop), id);
-        widget = makeValuePaddingWithHeading_(heading = makeHeading_Widget(label), as_Widget(drop));
+        if (argLabel_Command(spec, "noheading")) {
+            widget = makeValuePadding_(as_Widget(drop));
+        }
+        else {
+            widget = makeValuePaddingWithHeading_(heading = makeHeading_Widget(label), as_Widget(drop));
+        }
         setCommandHandler_Widget(widget, dropdownHeadingHandler_);
         widget->padding[2] = gap_UI;
         setUserData_Object(widget, drop);
@@ -705,7 +733,8 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
         }
         setId_Widget(as_Widget(input), id);
         setUrlContent_InputWidget(input, argLabel_Command(spec, "url"));
-        setSelectAllOnFocus_InputWidget(input, argLabel_Command(spec, "selectall"));        
+        setLineBreaksEnabled_InputWidget(input, !argLabel_Command(spec, "nolinebreaks"));
+        setSelectAllOnFocus_InputWidget(input, argLabel_Command(spec, "selectall"));
         setFont_InputWidget(input, labelFont_());
         if (argLabel_Command(spec, "noheading")) {
             widget = makeValuePadding_(as_Widget(input));
@@ -713,7 +742,7 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
         }
         else {
             setFlags_Widget(as_Widget(input), alignRight_WidgetFlag, iTrue);
-            setContentPadding_InputWidget(input, 3 * gap_UI, 0);
+            setContentPadding_InputWidget(input, 0 * gap_UI, 0);
             if (hasLabel_Command(spec, "unit")) {
                 iWidget *unit = addChildFlags_Widget(
                     as_Widget(input),
@@ -738,12 +767,23 @@ void makePanelItem_Mobile(iWidget *panel, const iMenuItem *item) {
         updateItems_CertListWidget(certList);
         invalidate_ListWidget(list);
     }
+    else if (equal_Command(spec, "snippetlist")) {
+        iSnippetWidget *snipList = new_SnippetWidget();
+        setBackgroundColor_Widget(as_Widget(list_SnippetWidget(snipList)),
+                                  uiBackgroundSidebar_ColorId);
+        widget = as_Widget(snipList);
+        iRelease(removeChild_Widget(widget, findChild_Widget(widget, "sniped.new")));
+        setFlags_Widget(widget, borderTop_WidgetFlag | borderBottom_WidgetFlag, iTrue);
+    }
     else if (equal_Command(spec, "button")) {
         widget = as_Widget(heading = makePanelButton_(label, item->command));
         setFlags_Widget(widget, selected_WidgetFlag, argLabel_Command(spec, "selected") != 0);
     }
-    else if (equal_Command(spec, "navi.action")) {
-        iLabelWidget *action = new_LabelWidget(label, item->command);
+    else if (equal_Command(spec, "navi.action") ||
+             equal_Command(spec, "navi.menubutton")) {
+        iLabelWidget *action = equal_Command(spec, "navi.action")
+            ? new_LabelWidget(label, item->command)
+            : makeMenuButton_LabelWidget(label, item->data, iInvalidSize);
         setId_Widget(as_Widget(action), id);
         setFlags_Widget(as_Widget(action),
                         hidden_WidgetFlag | collapse_WidgetFlag | frameless_WidgetFlag |
@@ -803,12 +843,19 @@ void makePanelItems_Mobile(iWidget *panel, const iMenuItem *itemsNullTerminated)
     }
 }
 
+static iBool isCancelAction_(const iMenuItem *item) {
+    return !iCmpStr(item->label, "${cancel}") || !iCmpStr(item->label, "${close}");
+}
+
 static const iMenuItem *findDialogCancelAction_(const iMenuItem *items, size_t n) {
-    if (n <= 1) {
+    if (n == 0) {
         return NULL;
     }
+//    if (n == 1) {
+//        return isCancelAction_(&items[0]) ? &items[0] : NULL;
+//    }
     for (size_t i = 0; i < n; i++) {
-        if (!iCmpStr(items[i].label, "${cancel}") || !iCmpStr(items[i].label, "${close}")) {
+        if (isCancelAction_(&items[i])) {
             return &items[i];
         }
     }
@@ -842,7 +889,8 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
     setFlags_Widget(panels,
                     resizeToParentWidth_WidgetFlag |
                         (isFullHeight
-                             ? resizeToParentHeight_WidgetFlag | leftEdgeDraggable_WidgetFlag |
+                             ? hittable_WidgetFlag |
+                                   resizeToParentHeight_WidgetFlag | leftEdgeDraggable_WidgetFlag |
                                    horizontalOffset_WidgetFlag
                              : (arrangeHeight_WidgetFlag | moveToParentBottomEdge_WidgetFlag)) |
                         frameless_WidgetFlag | focusRoot_WidgetFlag | commandOnClick_WidgetFlag,
@@ -928,9 +976,7 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
         checkIcon_LabelWidget(naviBack);
         setId_Widget(as_Widget(naviBack), "panel.back");
         setFont_LabelWidget(naviBack, labelFont_());
-        //if (!isFullHeight) {
-            setTextColor_LabelWidget(naviBack, uiTextAction_ColorId);
-        //}
+        setTextColor_LabelWidget(naviBack, uiTextAction_ColorId);
         naviActions = addChildFlags_Widget(
             navi,
             iClob(new_Widget()),
@@ -940,7 +986,7 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
         addChildFlags_Widget(panels, iClob(navi),
                              (isFullHeight ? drawBackgroundToVerticalSafeArea_WidgetFlag : 0) |
                                  arrangeHeight_WidgetFlag | //resizeWidthOfChildren_WidgetFlag |
-                                 resizeToParentWidth_WidgetFlag | arrangeVertical_WidgetFlag);    
+                                 resizeToParentWidth_WidgetFlag | arrangeVertical_WidgetFlag);
     }
     iBool haveDetailPanels = iFalse;
     /* Create panel contents based on provided items. */
@@ -952,7 +998,7 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
             const iString *label = hasLabel_Command(item->label, "text")
                                        ? collect_String(suffix_Command(item->label, "text"))
                                        : collectNewFormat_String("${%s}", id);
-            iLabelWidget * button =
+            iLabelWidget *button =
                 addChildFlags_Widget(topPanel,
                                      iClob(makePanelButton_(cstr_String(label), "panel.open")),
                                      borderTop_WidgetFlag);
@@ -965,6 +1011,9 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
             if (argLabel_Command(item->label, "noscroll")) {
                 setFlags_Widget(panel, overflowScrollable_WidgetFlag, iFalse);
             }
+            if (hasLabel_Command(item->label, "buttonid")) {
+                setId_Widget(as_Widget(button), cstr_Command(item->label, "buttonid"));
+            }
             makePanelItems_Mobile(panel, item->data);
         }
         else {
@@ -983,7 +1032,7 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
             setFlags_Widget(as_Widget(naviBack), alignLeft_WidgetFlag, iFalse);
             setFlags_Widget(as_Widget(naviBack), alignRight_WidgetFlag, iTrue);
             setIcon_LabelWidget(naviBack, 0);
-            setFont_LabelWidget(naviBack, labelBoldFont_());            
+            setFont_LabelWidget(naviBack, labelBoldFont_());
         }
         else if (defaultItem && defaultItem != cancelItem) {
             if (!haveDetailPanels) {
@@ -1042,7 +1091,7 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
 
 /*
          Landscape Layout                 Portrait Layout
-                                      
+
 ┌─────────┬──────Detail─Stack─────┐    ┌─────────┬ ─ ─ ─ ─ ┐
 │         │┌───────────────────┐  │    │         │Detail
 │         ││┌──────────────────┴┐ │    │         │Stack    │
@@ -1057,13 +1106,13 @@ void initPanels_Mobile(iWidget *panels, iWidget *parentWidget,
 │         │  └───────────────────┘│    │         │ └──────┘
 └─────────┴───────────────────────┘    └─────────┴ ─ ─ ─ ─ ┘
                                                   underneath
- 
+
 In portrait, top panel and detail stack are all stacked together.
 */
 
 void setupMenuTransition_Mobile(iWidget *sheet, iBool isIncoming) {
     if (!isUsingPanelLayout_Mobile()) {
-        return;    
+        return;
     }
     const iBool isHorizPanel = (flags_Widget(sheet) & horizontalOffset_WidgetFlag) != 0;
     if (isHorizPanel && isLandscape_App()) {
@@ -1086,6 +1135,10 @@ void setupMenuTransition_Mobile(iWidget *sheet, iBool isIncoming) {
 }
 
 void setupSheetTransition_Mobile(iWidget *sheet, int flags) {
+    if (isPromoted_Widget(sheet) && isExtraWindowSizeInfluencer_Widget(sheet)) {
+        /* This has been promoted to a window, shouldn't animate it. */
+        return;
+    }
     const iBool isIncoming = (flags & incoming_TransitionFlag) != 0;
     const int   dir        = flags & dirMask_TransitionFlag;
     if (!isUsingPanelLayout_Mobile()) {
@@ -1122,7 +1175,7 @@ void setupSheetTransition_Mobile(iWidget *sheet, int flags) {
         }
         setVisualOffset_Widget(sheet, 0, deviceType_App() == tablet_AppDeviceType ? 350 : 275,
                                easeOut_AnimFlag | softer_AnimFlag);
-    }        
+    }
     else {
         switch (dir) {
             case right_TransitionDir: {
@@ -1186,3 +1239,59 @@ int bottomSafeInset_Mobile(void) {
     return 0;
 #endif
 }
+
+#if !defined (iPlatformAppleMobile)
+iBool isSupported_SystemMenu(void) {
+    return iFalse;
+}
+
+iBool makePopup_SystemMenu(iWidget *owner) {
+    iUnused(owner);
+    return iFalse;
+}
+
+void setRect_SystemMenu(iWidget *owner, iRect anchorRect) {
+    iUnused(owner, anchorRect);
+}
+
+void setHidden_SystemMenu(iWidget *owner, iBool hide) {
+    iUnused(owner, hide);
+}
+
+void updateItems_SystemMenu(iWidget *owner, const iMenuItem *items, size_t n) {
+    iUnused(owner);
+}
+
+void releasePopup_SystemMenu(iWidget *owner) {
+    iUnused(owner);
+}
+#endif /* !iPlatformAppleMobile */
+
+void updateAfterBoundsChange_SystemMenu(iWidget *owner) {
+    iAssert(isSupported_SystemMenu());
+    //printf("updating bounds of sysmenu owner %p\n", owner);
+    iAssert(flags_Widget(owner) & nativeMenu_WidgetFlag);
+    iWidget *parent = parent_Widget(owner);
+    if (isInstance_Object(parent, &Class_LabelWidget)) {
+        const iWidget *menuFocusRoot   = focusRoot_Widget(parent);
+        const iWidget *activeFocusRoot = focusRoot_Widget(root_Widget(parent));
+//        printf("--- menuFocusRoot: ");
+//        identify_Widget(menuFocusRoot);
+//        printf("--- activeFocusRoot: ");
+//        identify_Widget(activeFocusRoot);    
+        if (!isVisible_Widget(parent) || isDisabled_Widget(parent) ||
+            /* other focus root blocks the parent? */
+            (menuFocusRoot != activeFocusRoot &&
+             !(hasParent_Widget(menuFocusRoot, activeFocusRoot) ||
+               userData_Object(parent) == activeFocusRoot /* navigation bar buttons */))) {
+            setHidden_SystemMenu(owner, iTrue);
+        }
+        else {
+            setRect_SystemMenu(owner, bounds_Widget(parent));
+            setHidden_SystemMenu(owner, iFalse);
+        }
+    }
+    else {
+        /* This is probably a submenu so it's not attached to any onscreen button. */
+    }
+}
