@@ -76,6 +76,7 @@ void        setupMenuTransition_Mobile  (iWidget *menu,  iBool isIncoming);
 void        setupSheetTransition_Mobile (iWidget *sheet, int flags);
 
 int         leftSafeInset_Mobile        (void);
+int         rightSafeInset_Mobile       (void);
 int         topSafeInset_Mobile         (void);
 int         bottomSafeInset_Mobile      (void);
 
@@ -98,10 +99,22 @@ iDeclareTypeConstructionArgs(SystemTextInput, iRect rect, int flags)
 
 void    setRect_SystemTextInput  (iSystemTextInput *, iRect rect);
 void    setText_SystemTextInput  (iSystemTextInput *, const iString *text, iBool allowUndo);
+void    insert_SystemTextInput   (iSystemTextInput *, const iString *text);
 void    setFont_SystemTextInput  (iSystemTextInput *, int fontId);
 void    setTextChangedFunc_SystemTextInput
         (iSystemTextInput *, void (*textChangedFunc)(iSystemTextInput *, void *), void *);
-void    selectAll_SystemTextInput(iSystemTextInput *);
+void    setSelection_SystemTextInput(iSystemTextInput *, iRangei selection);
+void    selectAll_SystemTextInput   (iSystemTextInput *);
 
 const iString * text_SystemTextInput    (const iSystemTextInput *);
 int     preferredHeight_SystemTextInput (const iSystemTextInput *);
+
+/*----------------------------------------------------------------------------------------------*/
+
+iBool   isSupported_SystemMenu  (void);
+iBool   makePopup_SystemMenu    (iWidget *owner);
+void    setRect_SystemMenu      (iWidget *owner, iRect anchorRect);
+void    setHidden_SystemMenu    (iWidget *owner, iBool hide);
+void    updateAfterBoundsChange_SystemMenu (iWidget *owner);
+void    updateItems_SystemMenu  (iWidget *owner, const iMenuItem *items, size_t n);
+void    releasePopup_SystemMenu (iWidget *owner);
