@@ -20,21 +20,22 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
+#define WIN32_LEAN_AND_MEAN
+#include <Windows.h>
+#include <windowsx.h>
+#include <dwmapi.h>
+#include <d2d1.h>
+
 #include "win32.h"
 #include "ui/window.h"
 #include "ui/command.h"
 #include "prefs.h"
 #include "app.h"
 
+#include <the_Foundation/path.h>
 #include <the_Foundation/sortedarray.h>
 #include <SDL_syswm.h>
 
-#define WIN32_LEAN_AND_MEAN
-#include <Windows.h>
-#include <windowsx.h>
-#include <dwmapi.h>
-#include <d2d1.h>
-
 static HWND windowHandle_(SDL_Window *win) {
     SDL_SysWMinfo wmInfo;
     SDL_VERSION(&wmInfo.version);
@@ -56,7 +57,7 @@ but for now this is what we have to do to avoid having a white title bar in dark
 
 Calling random functions from system DLLs is a great way to introduce crashes in the
 future! Be on the lookout for launch problems down the road.
-   
+
 Adapted from https://github.com/ysc3839/win32-darkmode. */
 
 enum WINDOWCOMPOSITIONATTRIB {
@@ -182,7 +183,7 @@ static iBool refreshTitleBarThemeColor_(HWND hwnd) {
 }
 
 static void enableDarkMode_Win32(void) {
-    RtlGetNtVersionNumbersFunc RtlGetNtVersionNumbers_ = 
+    RtlGetNtVersionNumbersFunc RtlGetNtVersionNumbers_ =
         (RtlGetNtVersionNumbersFunc)
         GetProcAddress(GetModuleHandleW(L"ntdll.dll"), "RtlGetNtVersionNumbers");
     if (!RtlGetNtVersionNumbers_) {
@@ -198,7 +199,7 @@ static void enableDarkMode_Win32(void) {
     }
     HMODULE hUxtheme = LoadLibraryExW(L"uxtheme.dll", NULL, LOAD_LIBRARY_SEARCH_SYSTEM32);
     if (hUxtheme) {
-		AllowDarkModeForWindow_ = (AllowDarkModeForWindowFunc) 
+		AllowDarkModeForWindow_ = (AllowDarkModeForWindowFunc)
             GetProcAddress(hUxtheme, MAKEINTRESOURCEA(133));
         AllowDarkModeForAppFunc AllowDarkModeForApp_ = NULL;
         SetPreferredAppModeFunc SetPreferredAppMode_ = NULL;
@@ -219,8 +220,12 @@ static void enableDarkMode_Win32(void) {
 }
 
 void init_Win32(void) {
+#if !SDL_VERSION_ATLEAST(2, 24, 0)
+    /* New SDL versions configure DPI awareness for us. */
     SetProcessDPIAware();
+#endif
     enableDarkMode_Win32();
+    RegisterApplicationRestart(L"", ~RESTART_NO_PATCH);
 }
 
 float desktopDPI_Win32(void) {
@@ -239,6 +244,27 @@ float desktopDPI_Win32(void) {
     return ratio;
 }
 
+iString *windowsDirectory_Win32(void) {
+    WCHAR winDir[MAX_PATH];
+    GetWindowsDirectoryW(winDir, MAX_PATH);
+    return newUtf16_String(winDir);
+}
+
+iString *tempDirectory_Win32(void) {
+    /* Calling GetTempPathW would just return C:\WINDOWS? A local config issue? */
+    WCHAR buf[32768];
+    if (GetEnvironmentVariableW(L"TMP", buf, sizeof(buf))) {
+        return newUtf16_String(buf);
+    }
+    if (GetEnvironmentVariableW(L"TEMP", buf, sizeof(buf))) {
+        return newUtf16_String(buf);
+    }
+    if (GetEnvironmentVariableW(L"USERPROFILE", buf, sizeof(buf))) {
+        return concatCStr_Path(collect_String(newUtf16_String(buf)), "AppData\\Local\\Temp");
+    }
+    return concatCStr_Path(collect_String(windowsDirectory_Win32()), "Temp");
+}
+
 void useExecutableIconResource_SDLWindow(SDL_Window *win) {
     HINSTANCE handle = GetModuleHandle(NULL);
     HICON icon = LoadIcon(handle, "IDI_ICON1");
@@ -253,25 +279,27 @@ void enableDarkMode_SDLWindow(SDL_Window *win) {
         HWND hwnd = windowHandle_(win);
         AllowDarkModeForWindow_(hwnd, TRUE);
         refreshTitleBarThemeColor_(hwnd);
-    }    
+    }
 }
 
 void handleCommand_Win32(const char *cmd) {
-    if (equal_Command(cmd, "theme.changed")) {        
-        iConstForEach(PtrArray, iter, mainWindows_App()) {
-            iMainWindow *mw = iter.ptr;
-            SDL_Window *win = mw->base.win;
+    if (equal_Command(cmd, "theme.changed")) {
+        iConstForEach(PtrArray, iter, regularWindows_App()) {
+            iWindow *w = iter.ptr;
+            SDL_Window *win = w->win;
             if (refreshTitleBarThemeColor_(windowHandle_(win)) &&
-                !isFullscreen_MainWindow(mw) &&
+                (type_Window(w) != main_WindowType ||
+                 !isFullscreen_MainWindow(as_MainWindow(w))) &&
                 !argLabel_Command(cmd, "auto")) {
-                /* Silly hack, but this will ensure that the non-client area is repainted. */
+                /* Silly hack, but this will ensure that the non-client
+                   area is repainted. */
                 SDL_MinimizeWindow(win);
                 SDL_RestoreWindow(win);
             }
         }
     }
     else if (equal_Command(cmd, "window.focus.gained")) {
-        /* Purge old windows from the darkness. */ 
+        /* Purge old windows from the darkness. */
         cleanDark_();
     }
 }
@@ -353,11 +381,11 @@ void processNativeEvent_Win32(const struct SDL_SysWMmsg *msg, iWindow *window) {
             if (wp == VK_RWIN) {
                 winDown_[1] = iFalse;
             }
-            break;            
+            break;
         }
         case WM_NCLBUTTONDBLCLK: {
             iMainWindow *mw = as_MainWindow(window);
-            POINT point = { GET_X_LPARAM(msg->msg.win.lParam), 
+            POINT point = { GET_X_LPARAM(msg->msg.win.lParam),
                             GET_Y_LPARAM(msg->msg.win.lParam) };
             ScreenToClient(hwnd, &point);
             iInt2 pos = init_I2(point.x, point.y);
@@ -379,7 +407,7 @@ void processNativeEvent_Win32(const struct SDL_SysWMmsg *msg, iWindow *window) {
         }
 #if 0
         case WM_NCLBUTTONUP: {
-            POINT point = { GET_X_LPARAM(msg->msg.win.lParam), 
+            POINT point = { GET_X_LPARAM(msg->msg.win.lParam),
                             GET_Y_LPARAM(msg->msg.win.lParam) };
             printf("%d,%d\n", point.x, point.y); fflush(stdout);
             ScreenToClient(hwnd, &point);
@@ -395,7 +423,7 @@ void processNativeEvent_Win32(const struct SDL_SysWMmsg *msg, iWindow *window) {
            However, the only useful function in the menu would be moving-via-keyboard,
            but that doesn't work with a custom frame. We could show a custom system menu? */
         case WM_NCRBUTTONUP: {
-            POINT point = { GET_X_LPARAM(msg->msg.win.lParam), 
+            POINT point = { GET_X_LPARAM(msg->msg.win.lParam),
                             GET_Y_LPARAM(msg->msg.win.lParam) };
             HMENU menu = GetSystemMenu(hwnd, FALSE);
             printf("menu at %d,%d menu:%p\n", point.x, point.y, menu); fflush(stdout);
